********************************************************************
* SAVE dweeb for dwload
*
* 2014 Tormod Volden
*
* lwasm --pragma=condundefzero -r -b -o/tmp/save.bin save.asm -DD32ROM  -l > /tmp/save.lst
********************************************************************


* dwload temporary RAM storage
namebuf  equ   $1d1		fixed by BASIC read filename function
startbuf equ   namebuf+6	write buffer, can reuse name buffer
endbuf   equ   startbuf+256
wdnum    equ   startbuf-4

* ROM routines used
OUTCH    equ   $BCAB		print char A to screen
OUTCHR   equ   $B54A		print char A to DEVNUM
OUTSTR   equ   $90E5		print string at X+1 to DEVNUM
OUTNUM   equ   $957A		print number in D to DEVNUM
GETFNAM  equ   $B7AA		read file name and length into $1d1
OKPRMPT  equ   $8371		BASIC OK prompt, command loop
IOERROR  equ   $B84B		IO_error
SYSERR   equ   $8344		system error
BACKSPC  equ   $9A89		print a backspace to DEVNUM

IntMasks equ   $50		disable IRQ and FIRQ

         IFNDEF RAMORG
RAMORG   equ   $7400
         ENDC

         org   RAMORG
Entry

* Get DW routine addresses from calling DWLOAD

         ldu   ,s               get stacked PC
         ldx   1,u
         stx   DWReadv
         ldx   3,u
         stx   DWWritev

         orcc  #IntMasks	disable FIRQ, IRQ

* Get file name from command line
         jsr   <$a5		peek at next character
         beq   noname		end of line peeked
         jsr   GETFNAM		read file name and length into $1d1 = namebuf
* Create named object on DW server
         ldx   #namebuf-1	packet buffer start
         clr   ,x		zero MSB for Y
         ldy   ,x++		length of file name (16 bit)
         lbeq  noname		no file name given?
         ldb   -1,x		length of name (8 bit)
         clr   b,x		zero terminate name string (for error)
         inc   ,--x		1 = DriveWire OP_NAMEOBJ_MOUNT
         inc   ,x		2 = DriveWire OP_NAMEOBJ_CREATE
         leay  2,y		length of DW packet, name length + 2
         jsr   [DWWritev]
* read back attributed drive number
         ldx   #startbuf-4	Our drive number variable
         clr   ,x
         leay  1,y		read one byte back (Y is 0 after DWWrite)
         jsr   [DWReadv]	get drive number
         tst   ,x+
         bne   WriteBas		successful create request
         ldb   #$19		MO ERROR
err      jmp   SYSERR		system error
noname   ldb   #$08		FC ERROR if missing filename
         bra   err

* named object has been created, proceed to save file
WriteBas
* prepare DriveWire Write packet
	lda   #$57		* OP_WRITE
	sta   -2,x
	clr   ,x+		* LSN bits 23-16
	clr   ,x+		* LSN bits 15-8
	clr   ,x+		* LSN bits 7-0
	tfr   x,u		* buffer pointer

* prepare DRAGON DOS BASIC header
	ldu   #startbuf
	lda   #$55		* Dragon DOS file header magic
	sta   ,u+
	lda   #1		* Dragon DOS Filetype 0x01 = BAS
	sta   ,u+
        ldx   <$19		* load address (not used when loading)
	stx   ,u++
* calculate length
	ldd   <$1b
	subd  <$19
	std   ,u++		* length
	stx   ,u++		* exec address (not used for BAS files)
	lda   #$AA
	sta   ,u+		* DRAGON DOS file header end marker

* checksum of BAS header into X
	lda   #9
	ldx   #0
	leay  -9,u
hnext	ldb   ,y+
	abx
	deca
	bne   hnext

* copy BASIC program into write buffer
	ldy   -7,u		* start of BASIC program
copyw
	ldb   ,y+
        stb   ,u+
	abx  			* update checksum
	cmpy  <$1b		* end of BASIC program ?
	beq   endbas
	cmpu  #endbuf
	bne   copyw
	jsr   writeb
	bra   copyw
* zero fill remaining buffer
endbas	clrb
fillz	stb   ,u+
	cmpu  #endbuf
	bne   fillz
	jsr   writeb

	andcc #~IntMasks	* enable interrupts
	rts

* write full block to server
writeb	stx   ,u		* checksum
	ldu   #startbuf		* reset buffer pointer
	pshs  y
	leax  -5,u
	ldy   #256+7
        jsr   [DWWritev]
* get status byte from server
	ldx   #namebuf
	ldy   #1
	clra		* clear carry bit for BECKER
	jsr   [DWReadv]
	bcs   ioerr
	bne   ioerr
	tst   ,x		* 0 means checksum OK
	bne   ioerr
* update sector counter
	ldx   -2,u
	leax  1,x
	stx   -2,u
	ldx   #0		* reset checksum
	puls  y,pc
	
ioerr	jmp   IOERROR

DWReadv  fdb 0
DWWritev fdb 0

	end   Entry
